from googleapiclient.discovery import build
from google_auth_oauthlib.flow import InstalledAppFlow
from google.auth.transport.requests import Request
import pickle
import logging
logging.getLogger('googleapicliet.discovery_cache').setLevel(logging.ERROR)

try: from definitions_prod import *
except Exception as e: pass # Surpass error. Note: Create definitions_prod.py

class GoogleSheet:

    def __init__(self, sheet_id=None, credentials=None, token=None):
        self.sheet_id = sheet_id
        self.credentials = credentials
        self.token = token

    def setup_sheets_api(self):
        SCOPES = 'https://www.googleapis.com/auth/spreadsheets'

        creds = None
        # The file token.pickle stores the user's access and refresh tokens, and is
        # created automatically when the authorization flow completes for the first
        # time.
        if os.path.exists(appConfig.GOOGLE_SHEET_TOKENS):
            with open(appConfig.GOOGLE_SHEET_TOKENS, 'rb') as token:
                creds = pickle.load(token)
        # If there are no (valid) credentials available, let the user log in.
        if not creds or not creds.valid:
            if creds and creds.expired and creds.refresh_token:
                creds.refresh(Request())
            else:
                flow = InstalledAppFlow.from_client_secrets_file(appConfig.GOOGLE_SHEET_CREDENTIALS, SCOPES)
                creds = flow.run_local_server()
            # Save the credentials for the next run
            with open(appConfig.GOOGLE_SHEET_TOKENS, 'wb') as token:
                pickle.dump(creds, token)

        self.service = build('sheets', 'v4', credentials=creds)

    def call_sheets_api(self, range_name='Sheet1!A1'):
        RANGE_NAME = range_name
        result = self.service.spreadsheets().values().get(spreadsheetId=self.sheet_id, range=RANGE_NAME).execute()
        values = result.get('values', [])

        if not values:
            raise Exception('Empty response from google sheet call_sheets_api')
        else:
            return values

    def update_sheets_api(self, range='Sheet1!Q1:R1', values=None):
        if not values:
            return False
        if not isinstance(values[0], list):
            raise Exception('Must array 2 dimension')
        body = {'values': values}
        self.service.spreadsheets().values().append(spreadsheetId=self.sheet_id,
                                                    range=range,
                                                    valueInputOption='RAW', body=body).execute()

    def batch_update_api(self, values=None):
        batch_update_values_request_body = {
            'data': values,
            'value_input_option': 'RAW',
        }
        self.service.spreadsheets().values().batchUpdate(spreadsheetId=self.sheet_id,
                                                         body=batch_update_values_request_body).execute()

if __name__ == '__main__':
    ''' https://developers.google.com/sheets/api/quickstart/python
        pip install --upgrade google-api-python-client google-auth-httplib2 google-auth-oauthlib
     '''

    g = GoogleSheet('1zR7e6FwkwoPBV5i80qVuQpHEfItUfl3s0Lc8OUs5pHY')
    g.setup_sheets_api()
    # g.update_sheets_api(values=['Updated'])
    print(g.call_sheets_api('Sheet1!Q1'))
    print('INFO: Update successful.')
